import os
import pygame
from kivy.app import App
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.uix.label import Label
from kivy.uix.progressbar import ProgressBar
from kivy.uix.textinput import TextInput
from kivy.uix.screenmanager import ScreenManager, Screen
from kivy.uix.floatlayout import FloatLayout
from kivy.clock import Clock
from kivy.uix.image import Image
from kivy.core.window import Window
from kivy.uix.widget import Widget
from kivy.uix.video import Video
from kivy.graphics import Line
from kivy.uix.scrollview import ScrollView
from kivy.uix.checkbox import CheckBox
pygame.init()
pygame.mixer.init()

'''
Welkom bij Muse! De beste jukebox applicatie in de wereld. Muse is geprogrammeerd en ontworpen door XXXX met hulp van de volgende bronnen:
    -Stackoverflow.com
    -Wikipedia
    -kivy.org
    -discovery.cs.illinois.edu
    -The power of friendship
    -ChatGPT
Met deze tool kan je muziek beluisteren als nooit tevoren. Bekijk de toegevoegde comments voor meer info.'''

#grootte en achtergrondkleur van het scherm bepalen
Window.clearcolor = ('#040130')
Window.size = (900, 650)

#Dit is het hoofdscherm met de muziek op
class RockWindow(BoxLayout):
    def __init__(self):
        super().__init__(orientation='vertical', spacing=20)
        self.music_folder = './muziek'
        self.selected_genres = [] 
        self.music_files = self.get_music_files(self.music_folder)
        self.current_song = None
        self.is_playing = False
        self.waitlist = []
        self.progress_bar_interval = None  # To hold the progress bar update interval
        self.search_text = ""

        # Create UI
        self.create_ui()

    #Hier halen we de muziekbestanden op
    def get_music_files(self, folder):
        music_files = []
        for root, dirs, files in os.walk(folder):
            for file in files:
                print (file)
                if file.endswith('.wav') or file.endswith('.mp3'):
                    parent_folder = os.path.basename(root)
                    if not self.selected_genres or parent_folder in self.selected_genres:
                        music_files.append(os.path.splitext(file)[0])  #Zonder '.wav' na de titel
        print(sorted(music_files))
        return sorted(music_files)

    #Hier stellen we de interface op met alle knoppen
    def create_ui(self):

        genres = ["Soulpop", "Teenpop", "RnB", "Indie", "Rap", "Country", "AltRock", "Alternative", "Jazz", "Rock"]

        #Layout voor de filter checkboxes
        checkboxlayout = BoxLayout(orientation='vertical', spacing=5, size_hint_x=0.2, size_hint_y=1)

        #Deze functie bepaald wat er gebeurt als je filtert per genre
        def on_checkbox_active(checkbox, value, genre):
            if value:
                self.selected_genres.append(genre)
            else:
                self.selected_genres.remove(genre)
            print("Selected genres:", self.selected_genres)
            self.update_song_buttons()

        for genre in genres:
            horiz_layout = BoxLayout(orientation='horizontal')
            checkbox = CheckBox()
            checkbox.bind(active=lambda checkbox, value, genre=genre: on_checkbox_active(checkbox, value, genre))
            label = Label(text=genre)
            horiz_layout.add_widget(checkbox)
            horiz_layout.add_widget(label)
            checkboxlayout.add_widget(horiz_layout)

        #Search bar bovenaan
        self.search_input = TextInput(hint_text="Search songs", multiline=False, size_hint=(1, None), height=50, background_color=(.9,.9,.9,1))
        self.search_input.bind(text=self.on_search)
        self.add_widget(self.search_input)
        

        #We maken een grote layout voor het scherm
        screen_layout=BoxLayout(orientation='horizontal')
        self.add_widget(screen_layout)

        #Layout van de wachtlijst
        self.waitlist_layout = BoxLayout(orientation='vertical', spacing=5, pos_hint={'x':0.50}, size_hint_x=(0.5))

        #Layout voor de muziekknoppen en 'add' knoppen
        self.button_layout = BoxLayout(orientation='vertical', spacing=5, size_hint_x=0.7, height=36*len(self.music_files), size_hint_y=None)
        self.update_song_buttons()

        #We maken knoppen voor elk liedje
        for music_file in self.music_files:
            if self.search_text is None or self.search_text.lower() in music_file.lower():
                full_button_layout = BoxLayout(orientation='horizontal', spacing=5, height=36*len(self.music_files))
                button = Button(text=music_file, on_press=self.play_music, size_hint_y=None, height=30, background_color=('#FF81CC'))
                add_button = Button(text="Add", on_press=lambda instance, name=music_file: self.add_to_waitlist(name), size_hint_y=None, height=button.height, width=50)

        #We steken ze in een scrollbare layout
        scroll_view = ScrollView(size_hint=(1, 1), do_scroll_y=True, do_scroll_x=False)
        scroll_view.scroll_x = 1
        scroll_view.add_widget(self.button_layout)

        #We voegen alles toe aan de grotere layout
        screen_layout.add_widget(checkboxlayout)
        screen_layout.add_widget(scroll_view)
        screen_layout.add_widget(self.waitlist_layout)

        #Titel voor het afspelende liedje
        self.now_playing_label = Label(text="", size_hint_y=None, height=30)
        self.add_widget(self.now_playing_label)

        #Bar beneden aan het scherm
        self.progress_bar = ProgressBar(max=1, value=0, size_hint_y=None, height=10)
        self.progress_bar.background_color = (.4089, .2844, .2267)
        self.progress_bar.bar_color = (.5, .2, .1, 1)
        self.add_widget(self.progress_bar)

        #We maken play/pauze, en een next knop
        button_layout1 = BoxLayout(orientation='horizontal', size_hint_y=None, height=50, spacing=10, size_hint_x=.7, pos_hint={'x':.25})
        self.play_pause_button = Button(text="Choose Song", on_press=self.toggle_music, size_hint_x=0.4, size_hint_y=None, height=50, background_color=('#FF81CC'))
        self.next_song_button = Button(text="Next", on_press=self.play_next_song, size_hint_x=0.15, size_hint_y=None, height=50, background_color=('#B95DA0'))              
        button_layout1.add_widget(self.play_pause_button)        
        button_layout1.add_widget(self.next_song_button)
        self.add_widget(button_layout1)

        #Tijdsduur van het afspelende liedje
        self.duration_label = Label(text="00:00 / 00:00", size_hint_y=None, height=30)
        self.add_widget(self.duration_label)

        #'Waitlist is empty' tekst
        self.empty_waitlist_label = Label(text="Waitlist is empty", size_hint=(None, None), size=(200, 50), pos_hint={'center_x': 0.5, 'center_y': 0.5})
        self.empty_waitlist_label.color = (1, 0, 0, 1)  
        self.empty_waitlist_label.opacity = 0 
        self.add_widget(self.empty_waitlist_label)


    def update_song_buttons(self):
        #Deze functie update de zichtbare knoppen adhv de zoektekst en de geselecteerde genres.
        self.button_layout.clear_widgets()
        search_count = 0
        for root, dirs, files in os.walk(self.music_folder):
            for file in files:
                if file.endswith('.wav') or file.endswith('.mp3'):
                    music_file = os.path.splitext(file)[0]
                    parent_folder = os.path.basename(root)
                    if (not self.selected_genres or parent_folder in self.selected_genres) and \
                       (not self.search_text or self.search_text.lower() in music_file.lower()):
                        full_button_layout = BoxLayout(orientation='horizontal', height=36)
                        button = Button(text=music_file, on_press=self.play_music, size_hint_y=None, height=30, background_color=('#FF81CC'))
                        add_button = Button(text="Add", on_press=lambda instance, name=music_file: self.add_to_waitlist(name), size_hint_x=None,size_hint_y=None, height=30, width=50, background_color=('#B95DA0'))
                        full_button_layout.add_widget(button)
                        full_button_layout.add_widget(add_button)
                        self.button_layout.add_widget(full_button_layout)
                        search_count += 1
        self.button_layout.height=(36*search_count)



    def on_search(self, instance, value):
        #Deze functie leest de zoektekst en activeert de bovenstaande
        self.search_text = value
        self.update_song_buttons()

    def play_music(self, button):
        #Deze functie speelt muziek af
        song_name = button.text
        file_path = self.find_music_file(song_name)

        print('Play music called')
        print(file_path)

        if file_path:
            if file_path == self.current_song:
                if self.is_playing:
                    pygame.mixer.music.pause()
                    self.is_playing = False
                    self.play_pause_button.text = 'Play' 
                else:
                    pygame.mixer.music.unpause()
                    self.is_playing = True
                    self.play_pause_button.text = 'Pause'
            else:
                self.current_song = file_path
                pygame.mixer.music.load(file_path)
                pygame.mixer.music.play()
                self.now_playing_label.text = "Now Playing: " + song_name

                sound = pygame.mixer.Sound(file_path)
                duration = sound.get_length()
                self.duration_label.text = self.format_time(0) + " / " + self.format_time(duration)

                self.progress_bar.value = 0
                Clock.unschedule(self.update_progress_bar)
                self.progress_bar_interval = Clock.schedule_interval(lambda dt: self.update_progress_bar(duration), 0.1)
                self.is_playing = True
                self.play_pause_button.text = 'Pause'
        else:
            print("File not found for song:", song_name)

    def find_music_file(self, song_name):
        for root, dirs, files in os.walk(self.music_folder):
            for file in files:
                if (file.endswith('.wav') or file.endswith('.mp3')) and os.path.splitext(file)[0] == song_name:
                    return os.path.join(root, file)
        return None


    def add_to_waitlist(self, song_name):
        # checkt of liedjes al in de wachtlijst zitten
        if song_name not in self.waitlist:
            self.waitlist.append(song_name)
            self.update_waitlist_ui()
        else:
            print("Song already in waitlist:", song_name)


    def update_waitlist_ui(self):
        self.waitlist_layout.clear_widgets()

        #voegt liedjes toe aan de wachtlijst
        for song_name in self.waitlist:
            label = Label(text=song_name, size_hint_y=None, height=30)
            self.waitlist_layout.add_widget(label)


    def play_next_song(self, instance):
        #Speelt het volgende liedje uit de wachtlijst.
        print("Play next song function called.")
        if self.waitlist:
            print("Waitlist is not empty.")
            # Play the first song in the waitlist
            self.play_selected_song(self.waitlist[0])
            # Remove the played song from the waitlist
            del self.waitlist[0]
            self.update_waitlist_ui()
        else:
            print("Waitlist is empty.")
            # No songs in the waitlist
            empty_label = Label(text="Waitlist is empty", color=(1, 0, 0, 1), size_hint_y=None, height=30)
            self.waitlist_layout.add_widget(empty_label)
            Clock.schedule_once(lambda dt: self.waitlist_layout.remove_widget(empty_label), 2)

    def play_selected_song(self, song_name):
        button_text = None
        for button_layout in self.button_layout.children:
            for button in button_layout.children:
                if button.text == song_name:
                    button_text = button.text
                    break
            if button_text:
                break

        if button_text:
            self.play_music(button)


    def toggle_music(self, instance):
        # Wisselt tussen play en pauze
        if self.current_song:
            if self.is_playing:
                pygame.mixer.music.pause()
                self.is_playing = False
                self.play_pause_button.text = 'Play'
            else:
                pygame.mixer.music.unpause()
                self.is_playing = True
                self.play_pause_button.text = 'Pause' 

    def update_progress_bar(self, duration):
        # Update de bar naarmate het liedje speelt
        if pygame.mixer.music.get_busy():
            pos = pygame.mixer.music.get_pos() / 1000.0 
            self.progress_bar.value = pos / duration
            self.duration_label.text = self.format_time(pos) + " / " + self.format_time(duration)
        else:
            self.progress_bar.value = 0

    def format_time(self, time_sec):
        minutes = int(time_sec) // 60
        seconds = int(time_sec) % 60
        return "{:02d}:{:02d}".format(minutes, seconds)

class HomeScreen(Screen):
    #Dit is het openingsscherm.
    music_paused = False
    def __init__(self, **kwargs):
        super(HomeScreen, self).__init__(**kwargs)
        self.img = Image(source=r'./muse_logo.png', size_hint=(3, 3), pos_hint={'center_x': 0.5, 'center_y': 0.5})
        self.add_widget(self.img)


        #De knop om de bibliotheek te openen
        self.rock_button = Button(text="Music Library", on_press=self.go_to_rock, size_hint_x=None, size_hint_y=None, width=500, height=70, background_color=('#FF81CC'),pos_hint={'center_x': 0.5, 'center_y': 0.28})
        self.add_widget(self.rock_button)

    def go_to_rock(self, instance):
        #De functie om dat daadwerkelijk te doen.
        self.manager.current = 'rock'

class RockScreen(Screen):
    def __init__(self, **kwargs):
        super(RockScreen, self).__init__(**kwargs)
        self.layout = BoxLayout(orientation='vertical')
        self.rock_window = RockWindow()
        self.layout.add_widget(self.rock_window)
        self.add_widget(self.layout)

        # De knop die terug naar home gaat
        self.back_button = Button(text="Back", on_press=self.go_back_home, size_hint=(None, None), size=(100, 50), pos_hint={'x': 0, 'y': 1})
        self.back_button.background_color = ('#B95DA0')
        self.rock_window.add_widget(self.back_button)

    def go_back_home(self, instance):
        pygame.mixer.music.pause()
        # De functie die je terug naar het thuisscherm bent
        self.manager.get_screen('home').music_paused = True  
        self.manager.current = 'home'



class MyApp(App):
    def build(self):
        self.title = "Muse"
        self.icon='./muse_note.png'
        sm = ScreenManager()
        sm.add_widget(HomeScreen(name='home'))
        sm.add_widget(RockScreen(name='rock'))
        return sm
        return HomeScreen()
        return RockScreen()

if __name__ == "__main__":
    app = MyApp()
    app.run()
